@@ -1,8 +1,10 @@
 import asyncio
 import dataclasses
 import re
+from logging import Logger, getLogger
 from typing import (  # type: ignore
     Any,
+    AsyncGenerator,
     ClassVar,
     Generic,
     List,
@@ -37,6 +39,8 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
     key_attrs: ClassVar[Sequence[str]]
     many_key_attrs: ClassVar[Sequence[str]]
     __skip_cls_validation__: Sequence[str] = ()
+    timeout: int = 1
+    logger: Logger = getLogger(__name__)
 
     def __init_subclass__(
         cls,
@@ -95,6 +99,39 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
     ) -> Optional[EntityData]:
         raise NotImplementedError()  # pragma: no cover
 
+    async def get_memory_data_timeout(
+        self, key: str, query: 'BaseQuery[Entity, EntityData, FallbackKey]',
+    ) -> Optional[EntityData]:
+        try:
+            return await asyncio.wait_for(
+                self.get_memory_data(key, query), self.timeout
+            )
+        except asyncio.TimeoutError:
+            self.logger.warning(
+                'skip memory_data; timeout for '
+                f'key={key}, timeout={self.timeout}'
+            )
+            return None
+
+    async def get_fallback_data_timeout(
+        self,
+        query: Union['BaseQuery[Entity, EntityData, FallbackKey]', Entity],
+        *,
+        for_memory: bool = False,
+    ) -> Optional[EntityData]:
+        try:
+            return await asyncio.wait_for(
+                self.get_fallback_data(query, for_memory=for_memory),
+                self.timeout,
+            )
+        except asyncio.TimeoutError:
+            self.logger.warning(
+                'skip fallback_data; timeout for '  # type: ignore
+                f'key={self.memory_key(query)}, '
+                f'timeout={self.timeout}'
+            )
+            raise
+
     def make_entity(
         self,
         data: EntityData,
@@ -123,7 +160,12 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
     def make_memory_data_from_entity(self, entity: Entity) -> EntityData:
         raise NotImplementedError()  # pragma: no cover
 
-    async def add_fallback(self, entity: Entity, *entities: Entity) -> None:
+    async def add_fallback(
+        self,
+        entity: Entity,
+        *entities: Entity,
+        fallback_ttl: Optional[int] = None,
+    ) -> None:
         raise NotImplementedError()  # pragma: no cover
 
     async def entity(
@@ -134,52 +176,73 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
         else:
             return await self.get_fallback(query)
 
-    async def entities(
+    def entities(
         self, query: 'QueryMany[Entity, EntityData, FallbackKey]',
-    ) -> List[Entity]:
-        tasks = []
-        entities = []
-
+    ) -> AsyncGenerator[Entity, None]:
         if query.memory:
-            for query_ in query.queries:
-                task = asyncio.create_task(self.get_memory(query_))
-                task.add_done_callback(task_done_callback)
-                tasks.append(task)
+            return self.get_memory_many(query)
         else:
-            for query_ in query.queries:
-                task = asyncio.create_task(self.get_fallback(query_))
-                task.add_done_callback(task_done_callback)
-                tasks.append(task)
+            return self.get_fallback_many(query)
 
-        for t in tasks:
+    async def get_fallback_many(
+        self, query: 'QueryMany[Entity, EntityData, FallbackKey]',
+    ) -> AsyncGenerator[Entity, None]:
+        for query_ in query.queries:
             try:
-                entities.append(await t)
+                yield await self.get_fallback(query_)
             except EntityNotFoundError:
                 continue
 
-        if not entities:
-            raise EntityNotFoundError(query)
+    async def get_memory_many(
+        self, query: 'QueryMany[Entity, EntityData, FallbackKey]',
+    ) -> AsyncGenerator[Entity, None]:
+        for query_i in query.queries:
+            memory_key = self.memory_key(query_i)
+            memory_data = await self.get_memory_data_timeout(
+                memory_key, query_i
+            )
 
-        return entities
+            if memory_data:
+                yield self.make_entity(memory_data, query_i)
+
+            elif not await self.already_got_not_found(query_i):
+                try:
+                    fallback_data = await self.get_fallback_data_timeout(
+                        query_i, for_memory=True
+                    )
+                except asyncio.TimeoutError:
+                    ...
+                else:
+                    if fallback_data is None:
+                        await self.set_fallback_not_found(query_i)
+                    else:
+                        memory_data = await self.add_memory_data_from_fallback(
+                            memory_key, query_i, fallback_data
+                        )
+                        await self.set_expire_time(memory_key)
+                        yield self.make_entity(memory_data, query_i)
 
     async def get_memory(
         self, query: 'Query[Entity, EntityData, FallbackKey]',
     ) -> Entity:
         memory_key = self.memory_key(query)
-        memory_data = await self.get_memory_data(memory_key, query)
+        memory_data = await self.get_memory_data_timeout(memory_key, query)
 
         if not memory_data and not await self.already_got_not_found(query):
-            fallback_data = await self.get_fallback_data(
-                query, for_memory=True
-            )
-
-            if fallback_data is None:
-                await self.set_fallback_not_found(query)
-            else:
-                memory_data = await self.add_memory_data_from_fallback(
-                    memory_key, query, fallback_data
+            try:
+                fallback_data = await self.get_fallback_data_timeout(
+                    query, for_memory=True
                 )
-                await self.set_expire_time(memory_key)
+            except asyncio.TimeoutError:
+                ...
+            else:
+                if fallback_data is None:
+                    await self.set_fallback_not_found(query)
+                else:
+                    memory_data = await self.add_memory_data_from_fallback(
+                        memory_key, query, fallback_data
+                    )
+                    await self.set_expire_time(memory_key)
 
         if not memory_data:
             raise EntityNotFoundError(query)
@@ -190,7 +253,10 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
         self,
         query: Union['BaseQuery[Entity, EntityData, FallbackKey]', Entity],
     ) -> Entity:
-        data = await self.get_fallback_data(query)
+        try:
+            data = await self.get_fallback_data_timeout(query)
+        except asyncio.TimeoutError as error:
+            raise EntityNotFoundError(query) from error
 
         if data is None:
             raise EntityNotFoundError(query)
@@ -202,13 +268,21 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
         *entities: Entity,
         memory: bool = True,
         query: Optional['BaseQuery[Entity, EntityData, FallbackKey]'] = None,
+        fallback_ttl: Optional[int] = None,
     ) -> None:
         if memory:
-            return await self.add_memory(*entities)
+            return await self.add_memory(*entities, fallback_ttl=fallback_ttl)
         else:
-            return await self.add_fallback(*entities)
+            return await self.add_fallback(
+                *entities, fallback_ttl=fallback_ttl
+            )
 
-    async def add_memory(self, entity: Entity, *entities: Entity) -> None:
+    async def add_memory(
+        self,
+        entity: Entity,
+        *entities: Entity,
+        fallback_ttl: Optional[int] = None,
+    ) -> None:
         memory_key = self.memory_key(entity)
 
         if await self.memory_data_source.exists(memory_key):
@@ -216,7 +290,7 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
             await self.add_memory_data(memory_key, memory_data)
             await self.set_expire_time(memory_key)
 
-        await self.add_fallback(entity)
+        await self.add_fallback(entity, fallback_ttl=fallback_ttl)
         await self.delete_fallback_not_found(entity)
 
     async def set_expire_time(self, key: str) -> None:
@@ -272,11 +346,19 @@ class MemoryRepository(Generic[Entity, EntityData, FallbackKey]):
     async def already_got_not_found(
         self, query: Union['Query[Entity, EntityData, FallbackKey]', Entity],
     ) -> bool:
-        return bool(
-            await self.memory_data_source.exists(
-                self.fallback_not_found_key(query)
+        key = self.fallback_not_found_key(query)
+        try:
+            return bool(
+                await asyncio.wait_for(
+                    self.memory_data_source.exists(key), self.timeout,
+                )
             )
-        )
+        except asyncio.TimeoutError:
+            self.logger.warning(
+                'skip already_got_not_found; timeout for '
+                f'key={key}, timeout={self.timeout}'
+            )
+            return True
 
     async def delete_fallback_not_found(
         self, query: Union['Query[Entity, EntityData, FallbackKey]', Entity],
